@@ -11,6 +11,9 @@
  * Created: 6 August 2008
  */
 #include "config.h"
+
+#if !HAVE_LIBCURL		/* Only needed if we're not using libcurl */
+
 #include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -62,3 +65,4 @@ EXPORT bool read_mail_headers(FILE *f, Dictionary headers)
   return ferror(f) == 0;
 }
 
+#endif /* !HAVE_LIBCURL */
