@@ -72,10 +72,10 @@
  *
  * Author: Bert Bos
  * Created: 2 Dec 1998
- * Version: $Id: hxpipe.c,v 1.7 2013-03-23 23:18:09 bbos Exp $
+ * Version: $Id: hxpipe.c,v 1.8 2013-07-25 21:10:21 bbos Exp $
  *
  **/
-#include <config.h>
+#include "config.h"
 #include <stdio.h>
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
