@@ -23,7 +23,7 @@
  *
  * Author: Bert Bos <bert@w3.org>
  * Created Sep 1997
- * Version: $Id: hxtoc.c,v 1.10 2014-10-06 17:32:35 bbos Exp $
+ * Version: $Id: hxtoc.c,v 1.11 2015-01-22 16:21:33 bbos Exp $
  *
  **/
 #include "config.h"
@@ -33,6 +33,7 @@
 #include <stdio.h>
 #include <time.h>
 #include <stdbool.h>
+#include <getopt.h>
 #if STDC_HEADERS
 # include <string.h>
 #else
@@ -77,6 +78,7 @@
 #define NO_EXPAND false
 #define KEEP_ANCHORS true
 #define REMOVE_ANCHORS false
+#define DONT_FLATTEN false
 
 #define INDENT " "				/* Amount to indent ToC per level */
 
@@ -86,6 +88,7 @@ static bool xml = false;			/* Use <empty /> convention */
 static bool bctarget = true;			/* Generate <a name=> after IDs */
 static string toc_class = "toc";		/* <ul class="..."> */
 static bool use_div = false;			/* Option -d */
+static bool do_flatten = false;			/* Option -f */
 
 
 /* handle_error -- called when a parse error occurred */
@@ -227,7 +230,7 @@ static bool first_child_is_heading(Tree t)
 }
 
 static void expand(Tree t, bool *write, bool exp, bool keep_anchors,
-		   int div_depth);
+		   int div_depth, bool flatten);
 
 /* toc -- create a table of contents */
 static void toc(Tree t, int *curlevel, bool *item_is_open, int div_depth)
@@ -288,7 +291,7 @@ static void toc(Tree t, int *curlevel, bool *item_is_open, int div_depth)
 	} else {
 	  printf("<li><a href=\"#%s\">", id);
 	}
-	expand(t, &write, NO_EXPAND, REMOVE_ANCHORS, div_depth);
+	expand(t, &write, NO_EXPAND, REMOVE_ANCHORS, div_depth, do_flatten);
 	printf("</a>");
 	*item_is_open = true;
       } else {
@@ -306,12 +309,12 @@ static void toc(Tree t, int *curlevel, bool *item_is_open, int div_depth)
 
 /* expand -- write the tree, inserting ID's at H* and inserting a toc */
 static void expand(Tree t, bool *write, bool exp, bool keep_anchors,
-		   int div_depth)
+		   int div_depth, bool flatten)
 {
   conststring val;
   Tree h;
   pairlist a;
-  string s;
+  conststring s;
   int level;
   bool item_is_open = false;
 
@@ -369,20 +372,32 @@ static void expand(Tree t, bool *write, bool exp, bool keep_anchors,
 	  if (!get_attrib(h, "id")) set_attrib(h, "id", gen_id(h));
 	}
 	if (*write) {
-	  if (! keep_anchors && eq(h->name, "a")) {
+	  if (flatten && ((s = get_attrib(h, "alt")))) {
+	    /* Flatten: use ALT attribute instead of element */
+	    printf("%s", s);
+	  } else if (flatten && !eq(h->name, "bdo")
+		     && ((s = get_attrib(h, "dir")))) {
+	    /* Flatten: keep DIR attributes */
+	    printf("<span dir=\"%s\">", s);
+	    expand(h, write, exp, false, div_depth, flatten);
+	    printf("</span>");
+	  } else if (flatten && !eq(h->name, "bdo")) {
+	    /* Flatten: remove all elements except BDO */
+	    expand(h, write, exp, false, div_depth, flatten);
+	  } else if (! keep_anchors && eq(h->name, "a")) {
 	    /* Don't write the <a> and </a> tags */
-	    expand(h, write, exp, false, div_depth);
+	    expand(h, write, exp, false, div_depth, flatten);
 	  } else if (! keep_anchors && eq(h->name, "span")
 		     && has_class(h->attribs, INDEX)) {
 	    /* Don't write <span.index>...</span> tags */
-	    expand(h, write, exp, false, div_depth);
+	    expand(h, write, exp, false, div_depth, flatten);
 	  } else if (! keep_anchors && eq(h->name, "dfn")) {
 	    /* Don't copy dfn tags to the ToC */
-	    expand(h, write, exp, false, div_depth);
+	    expand(h, write, exp, false, div_depth, flatten);
 	  } else if (eq(h->name, "a") && (has_class(h->attribs, TARGET)
 		     || has_class(h->attribs, TOC))) {
 	    /* This <a> was inserted by toc itself; remove it */
-	    expand(h, write, exp, false, div_depth);
+	    expand(h, write, exp, false, div_depth, flatten);
 	  } else {
 	    printf("<%s", h->name);
 	    for (a = h->attribs; a != NULL; a = a->next) {
@@ -401,7 +416,7 @@ static void expand(Tree t, bool *write, bool exp, bool keep_anchors,
 	      if (bctarget && is_mixed(h->name) && (val = get_attrib(h, "id"))
 		  && !eq(h->name, "a") && ! xml)
 		printf("<a class=\"%s\" name=\"%s\"></a>", TARGET, val);
-	      expand(h, write, exp, keep_anchors, div_depth);
+	      expand(h, write, exp, keep_anchors, div_depth, flatten);
 	      printf("</%s>", h->name);
 	    }
 	  }
@@ -426,9 +441,34 @@ static void usage(string name)
 
 int main(int argc, char *argv[])
 {
-  int i, status;
+  int c, status;
   bool write = true;
 
+  while ((c = getopt(argc, argv, "l:h:xtdc:f")) != -1) {
+    switch (c) {
+    case 'l': toc_low = atoi(optarg); break;
+    case 'h': toc_high = atoi(optarg); break;
+    case 'x': xml = true; break;
+    case 't': bctarget = false; break;
+    case 'd': use_div = true; break;
+    case 'c': toc_class = newstring(optarg); break;
+    case 'f': do_flatten = true; break;
+    default: usage(argv[0]);
+    }
+  }
+  if (toc_low < 1) toc_low = 1;
+  if (toc_high > 6) toc_high = 6;
+
+  if (argc > optind + 1) {
+    usage(argv[0]);
+  } else if (optind >= argc || eq(argv[optind], "-")) {
+    yyin = stdin;
+  } else if (!(yyin = fopenurl(argv[optind], "r", &status))) {
+    perror(argv[optind]); exit(2);
+  } else if (status != 200) {
+    errexit("%s : %s\n", argv[optind], http_strerror(status));
+  }
+
   /* Bind the parser callback routines to our handlers */
   set_error_handler(handle_error);
   set_start_handler(start);
@@ -441,38 +481,12 @@ int main(int argc, char *argv[])
   set_emptytag_handler(handle_emptytag);
   set_endtag_handler(handle_endtag);
 
-  yyin = stdin;
-  for (i = 1; i < argc; i++) {
-    if (eq(argv[i], "-l")) {
-      if (i >= argc - 1) usage(argv[0]);
-      toc_low = atoi(argv[++i]);
-    } else if (eq(argv[i], "-h")) {
-      if (i >= argc - 1) usage(argv[0]);
-      toc_high = atoi(argv[++i]);
-    } else if (eq(argv[i], "-x")) {
-      xml = true;
-    } else if (eq(argv[i], "-t")) {
-      bctarget = false;
-    } else if (eq(argv[i], "-d")) {
-      use_div = true;
-    } else if (eq(argv[i], "-")) {
-      /* yyin = stdin; */
-    } else if (eq(argv[i], "-c")) {
-      if (i >= argc - 1) usage(argv[0]);
-      toc_class = newstring(argv[++i]);
-    } else {
-      yyin = fopenurl(argv[i], "r", &status);
-      if (yyin == NULL) {perror(argv[i]); exit(2);}
-      if (status != 200) errexit("%s : %s\n", argv[i], http_strerror(status));
-    }
-  }
-  if (toc_low < 1) toc_low = 1;
-  if (toc_high > 6) toc_high = 6;
-
   if (yyparse() != 0) exit(3);
 
   tree = get_root(tree);
-  expand(tree, &write, EXPAND, KEEP_ANCHORS, 1);
+  expand(tree, &write, EXPAND, KEEP_ANCHORS, 1, DONT_FLATTEN);
+#if 0
   tree_delete(tree);				/* Just to test memory mgmt */
+#endif
   return 0;
 }
